(*****************************************************************************
 *          HOL-TestGen - specification based test case generation
 *                                                                            
 * max_test.thy - Testing List Properties
 * Copyright (C) 2005 Achim D. Brucker <brucker@inf.ethz.ch>   
 *                    Burkhart Wolff   <bwolff@inf.ethz.ch>    
 *                                                                            
 * This file is part of HOL-TestGen.         
 *                                                                            
 * HOL-TestGen is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)  
 * any later version.                                                         
 *                                                                            
 * HOL-TestGen is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
 * more details.                                                              
 *                                                                            
 * You should have received a copy of the GNU General Public License along    
 * with this program; if not, write to the Free Software Foundation, Inc.,    
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.                  
 ******************************************************************************)

header {* Introduction *}

theory max_test = Testing:

text{* This introductory example explains the standard 
       \testgen{} method resulting in a formalized test 
       plan that is documented in a machine-checked 
       text like this theory document.

       We declare the context of this document---which must be the
       theory ``Testing'' at least in order to include the \testgen{}
       system libraries---and the type of the program under test.  *}


consts prog :: "int \<Rightarrow> int \<Rightarrow> int"

text{* Assume we want to test a simple program computing the 
       maximum value of two integers. We start by writing our 
       test specification:
     *}

test_spec "(prog a b) = (max a b)"

  txt {*
      By applying @{term gen_test_cases} we bring the proof
      state into testing normal form (\TNF).
      (see~\cite{brucker.ea:symbolic:2005} for details). 
      *}

  apply(gen_test_cases 1 0 "prog" simp: max_def)

  txt {* 
      which leads to the test partitioning one would expect:
      @{subgoals [display, goals_limit=50]}.

      Now we bind the test theorem to a particular name
      in the test environment:
      *}

  store_test_thm "max_test"



text {* This concludes the test case generation phase.
      Now we turn to the test data generation, which
      is---based on standard configurations in the
      test environment to be discussed in later 
      examples---just the top-level command:
      *}

  gen_test_data "max_test" 

text {* The Isabelle command \texttt{thm} allows for
      interactive inspections of the result: *}

  thm max_test.test_data

text {* which is:

        @{thm [display] max_test.test_data} 

        in this case. *}

text {* Analogously, we can also inspect the test hypotheses 
        and the test theorem: *}

  thm max_test.test_hyps

text {* which yields: 
 
        @{thm [display] max_test.test_hyps} 

        and
      *}

  thm max_test.test_thm

text {* resulting in:

        @{thm [display] max_test.test_thm} *}




text {* We turn now to the automatic generation of a test harness.
        This is performed by the top-level command: *}

gen_test_script "document/max_script.sml" "max_test" "prog" "myMax.max"

text {* which generates:
\lstinputlisting[style=sml]{max_script.sml}
*}
(*<*)
end
(*>*)
